@@ -27,11 +27,10 @@ Data model based on https://github.com/bramstein/typeset
 
 
 import re
-from monostyle.util.pos import PartofSpeech
+from monostyle.util.part_of_speech import PartofSpeech
 from monostyle.util.nodes import Node, LinkedList
 import monostyle.rst_parser.walker as rst_walker
-
-POS = PartofSpeech()
+from monostyle.util.fragment import FragmentBundle
 
 
 def reflow_trial(boxes, optimum, maximum, options):
@@ -109,19 +108,19 @@ def reflow_trial(boxes, optimum, maximum, options):
 
 def fix(root_rst, reports):
     """Search for the reports location in the full document."""
-    changes = []
+    changes = FragmentBundle()
     unlocated = reports.copy()
-    for node in rst_walker.iter_node(root_rst, ("text"), False):
+    for node in rst_walker.iter_node(root_rst, "text", False):
         if node.body.code:
             is_first_report = True
             reports_pro = []
             for report in unlocated:
-                if node.body.code.is_in_span(report.out.start_lincol):
+                if node.body.code.is_in_span(report.output.start_lincol):
                     reports_pro.append(report)
                     if is_first_report:
                         changes_para = reflow(node)
                         report.fix = changes_para
-                        changes.extend(changes_para)
+                        changes.combine(changes_para, check_align=False, merge=False)
                         is_first_report = False
 
             for report in reports_pro:
@@ -158,16 +157,16 @@ def reflow(node):
     maximum = maximum_abs - ind_block
     # print(show_limes(node, options["optimum"], options["maximum"]), end="\n")
 
-    changes_para = []
     boxes = process_para(node, ind_first - ind_block, options)
+    changes_para = FragmentBundle()
     if len(boxes) > 1:
         boxes = reflow_penalties(boxes, options)
         # show_demerits(boxes)
         boxes, lastbreak = reflow_trial(boxes, optimum, maximum, options)
         changes_para = stringify_space(boxes, lastbreak)
 
-    if len(changes_para) != 0:
-        changes_para = add_indent(changes_para, ind_block)
+        if not changes_para.is_empty():
+            changes_para = add_indent(changes_para, ind_block)
 
     return changes_para
 
@@ -178,7 +177,7 @@ def process_para(node, ind_offset, options):
         if node.child_nodes.is_empty():
             yield node
         else:
-            yield from rst_walker.iter_nodeparts(node, ("head", "body", "id_start"),
+            yield from rst_walker.iter_nodeparts(node, {"head", "body", "id_start"},
                                                  leafs_only=True)
 
     boxes = LinkedList()
@@ -215,20 +214,20 @@ def process_para(node, ind_offset, options):
 
 
 class Box(Node):
-    """Text container.
-
-    content -- text.
-    space -- following spaces as Fragment.
-    demerits -- weight of breaks after the Box.
-    extra_len -- length of whitespaces before the Box e.g. a first line indent.
-    """
+    """Text container."""
 
     __slots__ = (
         'content', 'space', 'demerits', 'totalpenalty', 'linkbreak',
         'best_linkbreak', 'word_len', 'space_len')
 
     def __init__(self, content, space, demerits, extra_len=0):
-        super(Box, self).__init__()
+        """
+        content -- text.
+        space -- following spaces as Fragment.
+        demerits -- weight of breaks after the Box.
+        extra_len -- length of whitespaces before the Box e.g. a first line indent.
+        """
+        super().__init__()
 
         self.content = content
         self.space = space
@@ -250,6 +249,7 @@ class Box(Node):
 
 def reflow_penalties(boxes, options):
     """Add spaces to ends of sentences and calculate @extra array of penalties."""
+    part_of_speech = PartofSpeech()
     pare_open_re = re.compile(r"[\(\[\{]")
     pare_close_re = re.compile(r"[\)\]\}]")
     pare_level = 0
@@ -278,9 +278,9 @@ def reflow_penalties(boxes, options):
 
         # Period or colon
         if punc_m := re.match(r"^([A-Za-z0-9-]+)[\"')]*([\.\:])[\"')]*$", box.content):
-            path = pos_weight(punc_m.group(1))
+            tag = pos_weight(part_of_speech, punc_m.group(1))
             # End of sentence
-            if (not path or path[0] != "abbreviation") or punc_m.group(2) == ":":
+            if (not tag or tag[0] != "abbreviation") or punc_m.group(2) == ":":
                 box.demerits += options["sentence"] / 2
                 if box.prev:
                     box.prev.demerits -= options["sentence"]
@@ -291,32 +291,40 @@ def reflow_penalties(boxes, options):
                     if options["dosentence2"] and box.next.next:
                         box.next.next.demerits -= options["sentence2"]
 
-            elif path:
+            elif tag:
                 # Don't break "Mr. X"
-                if path[0] == "abbreviation" and path[1] == "title":
+                if tag[0] == "abbreviation" and tag[1] == "title":
                     box.demerits -= options["namebreak"]
 
         # !? after word
         if (re.search(r"[\?\!][\"')]*$", box.content) and
                 (box.next and re.match(r"^[^A-Za-z]*[A-Z]", box.next.content))):
             box.demerits += options["sentence"] / 2
-            if box.prev: box.prev.demerits -= options["sentence"]
-            if box.next: box.next.demerits -= options["sentence"]
+            if box.prev:
+                box.prev.demerits -= options["sentence"]
+            if box.next:
+                box.next.demerits -= options["sentence"]
 
         if re.search(r"\,$", box.content): # Comma after word
             box.demerits += options["dependent"] / 2
-            if box.prev: box.prev.demerits -= options["dependent"]
-            if box.next: box.next.demerits -= options["dependent"]
+            if box.prev:
+                box.prev.demerits -= options["dependent"]
+            if box.next:
+                box.next.demerits -= options["dependent"]
 
         if re.search(r"[\;\"\'\)]$|--$", box.content): # Punctuation after word
             box.demerits += options["independent"] / 2
-            if box.prev: box.prev.demerits -= options["independent"]
-            if box.next: box.next.demerits -= options["independent"]
+            if box.prev:
+                box.prev.demerits -= options["independent"]
+            if box.next:
+                box.next.demerits -= options["independent"]
 
         if box.next and re.match(r"^\(", box.next.content): # Next word has opening parenthesis
             box.demerits += options["independent"] / 2
-            if box.prev: box.prev.demerits -= options["independent"]
-            if box.next: box.next.demerits -= options["independent"]
+            if box.prev:
+                box.prev.demerits -= options["independent"]
+            if box.next:
+                box.next.demerits -= options["independent"]
 
         if (box.next and re.match(r"[A-Z]", box.content) and
                 not re.search(r"\.", box.content) and re.match(r"[A-Z]", box.next.content)):
@@ -327,22 +335,25 @@ def reflow_penalties(boxes, options):
 
         if re.match(r"\d", box.content):
             box.demerits -= options["math"]
-            if box.prev: box.prev.demerits -= options["math"]
+            if box.prev:
+                box.prev.demerits -= options["math"]
 
         if single_char_m := re.match(r"\\?(.)$", box.content):
             if single_char_m.group(1) != "a":
                 box.demerits -= options["math"]
-                if box.prev: box.prev.demerits -= options["math"]
+                if box.prev:
+                    box.prev.demerits -= options["math"]
 
-        path = pos_weight(box.content)
-        if path and path[0] != "abbreviation":
-            box.demerits += path[-1][1] * options["connpenalty"]
-            if box.prev: box.prev.demerits += path[-1][0] * options["connpenalty"]
+        tag = pos_weight(part_of_speech, box.content)
+        if tag and tag[0] != "abbreviation":
+            box.demerits += tag[-1][1] * options["connpenalty"]
+            if box.prev:
+                box.prev.demerits += tag[-1][0] * options["connpenalty"]
 
     return boxes
 
 
-def pos_weight(word):
+def pos_weight(part_of_speech, word):
     """ Discourage a break have the value 1
     The value is the relative effort to avoid breaking
     a line before/after this part of speech tag,
@@ -380,6 +391,7 @@ def pos_weight(word):
             "distributive": [2, -2],
             "interrogative": [2, -2]
         },
+        "contraction": [0, -2],
         "abbreviation": {
             "title": {
                 "civil": [0, -4],
@@ -392,21 +404,21 @@ def pos_weight(word):
             "doc": [0, -3]
         }
     }
-    if path := POS.classify(word):
+    if tag := part_of_speech.tag(word):
         weight = weights.copy()
-        for seg in path:
+        for seg in tag:
             if seg in weight:
                 weight = weight[seg]
             else:
                 print("reflow weight key error: ", seg)
 
-        path.append(weight)
-        return path
+        tag.append(weight)
+        return tag
 
 
 def stringify_space(boxes, lastbreak):
     """Find changes turning spaces into newlines (or vice versa)."""
-    changes_para = []
+    changes_para = FragmentBundle()
     box = boxes.last()
     while box:
         is_last = True
@@ -419,12 +431,12 @@ def stringify_space(boxes, lastbreak):
                         new_content = []
                         for _ in range(max(1, nl_count)):
                             new_content.append('\n')
-                        box.space.content = new_content
-                        changes_para.append(box.space)
+                        box.space.replace_fill(new_content)
+                        changes_para.combine(box.space, check_align=False, merge=False)
                 is_last = False
             elif len(space_str) != 0 and (len(space_str) != 1 or space_str == '\n'):
-                box.space.content = ' '
-                changes_para.append(box.space)
+                box.space.replace_fill(' ')
+                changes_para.combine(box.space, check_align=False, merge=False)
             box = box.prev
 
         box = lastbreak
@@ -458,7 +470,8 @@ def stringify_word(boxes, lastbreak):
 def show_limes(node, optimum, maximum):
     """Visualize optimum and maximum."""
     lines_alt = []
-    for line in node.code.content:
+    for line in node.code.splitlines():
+        line = str(line)
         line = line[:-1] + ' ' * ((maximum - len(line)) + 1) + '\n'
         line = line[:optimum[0]] + '¦' + line[optimum[0]:optimum[1]]
         line += '¦' + line[optimum[1]:maximum] + '|' + line[maximum:]
@@ -484,41 +497,38 @@ def show_demerits(boxes):
 
 def measure_indent(node):
     """Measure the indent."""
-    is_first = True
     ind_re = re.compile(r"\A +")
     ind_first = 0
     ind_cur = 0
     ind_prev = None
-    for line in node.code.content:
-        if len(line.strip()) == 0:
-            is_first = False
+    for line in node.code.splitlines():
+        line_str = str(line)
+        if len(line_str.strip()) == 0:
             continue
 
         ind_cur = 0
-        if ind_m := re.match(ind_re, line):
+        if ind_m := re.match(ind_re, line_str):
             ind_cur = ind_m.end(0)
 
         if ind_prev and ind_prev != ind_cur:
             print("reflow: paragraph uneven indent")
 
-        if is_first:
+        if line.start_lincol[0] == node.code.start_lincol[0]:
             ind_first = ind_cur
             if not ind_m:
                 par_node = node.parent_node
-                while par_node and par_node.node_name in ("text", "body"):
+                while par_node and par_node.node_name in {"text", "body"}:
                     par_node = par_node.parent_node
 
                 # indent style
                 if (rst_walker.is_of(par_node,
-                        ("dir", "target", "substdef", "footdef", "citdef"), "*", "head")):
+                        {"dir", "target", "substdef", "footdef", "citdef"}, "*", "head")):
                     ind_cur = par_node.parent_node.name_start.code.start_lincol[1] + 3
                 else:
                     ind_cur = node.code.start_lincol[1]
         else:
             ind_prev = ind_cur
 
-        is_first = False
-
     return ind_first, ind_cur
 
 
@@ -527,7 +537,7 @@ def add_indent(changes_para, ind_block):
     ind_block_str = ' ' * ind_block
     for change in changes_para:
         if str(change) == '\n':
-            change.content.append(ind_block_str)
+            change.extend([ind_block_str], keep_end=True)
 
     return changes_para
 
