@@ -7,6 +7,7 @@ rst_parser.rst_node
 from monostyle.util.nodes import Node
 
 class NodeRST(Node):
+    """Universal node for RST."""
 
     __slots__ = ('node_name',
                  'indent', 'name_start', 'name', 'name_end', 'id_start', 'id', 'id_end',
@@ -15,7 +16,11 @@ class NodeRST(Node):
 
 
     def __init__(self, node_name, code):
-        super(NodeRST, self).__init__()
+        """
+        node_name -- node type.
+        code -- source of the node.
+        """
+        super().__init__()
 
         self.node_name = node_name
 
@@ -42,25 +47,28 @@ class NodeRST(Node):
 
 
     def append_part(self, name, code, is_full_line=False):
+        """Append a newly created part child."""
         if not is_full_line:
-            newpart = NodePartRST(name, code)
+            new_part = NodePartRST(name, code)
         else:
-            newpart = NodePartRST(name, None)
+            new_part = NodePartRST(name, None)
 
-        self.child_nodes.append(newpart)
+        self.child_nodes.append(new_part)
 
         if is_full_line:
-            newpart.append_code(code)
-        setattr(self, name, newpart)
+            new_part.append_code(code)
+        setattr(self, name, new_part)
 
 
     def insert_part(self, name, code, after):
-        newpart = NodePartRST(name, code)
-        self.child_nodes.insert_after(after, newpart)
-        setattr(self, name, newpart)
+        """Insert newly created part child after a reference node."""
+        new_part = NodePartRST(name, code)
+        self.child_nodes.insert_after(after, new_part)
+        setattr(self, name, new_part)
 
 
     def append_code(self, code):
+        """Extent the node's code."""
         if self.code is None:
             if code is None:
                 self.code = code
@@ -72,6 +80,7 @@ class NodeRST(Node):
 
 
     def prev_leaf(self):
+        """Return the previous leaf node."""
         if self.prev:
             return self.prev
         node = self
@@ -92,6 +101,7 @@ class NodeRST(Node):
 
 
     def next_leaf(self):
+        """Return the next leaf node."""
         if self.next:
             return self.next
         node = self
@@ -121,12 +131,17 @@ class NodeRST(Node):
 
 
 class NodePartRST(Node):
+    """Child node parts of RST nodes."""
 
     __slots__ = ('node_name', 'code', 'active', 'is_parsed', 'is_parsing')
 
 
     def __init__(self, node_name, code):
-        super(NodePartRST, self).__init__()
+        """
+        node_name -- node part type.
+        code -- source of the node.
+        """
+        super().__init__()
 
         self.node_name = node_name
 
@@ -139,21 +154,27 @@ class NodePartRST(Node):
         self.is_parsing = False
 
 
-    def append_child(self, newnode, prop_code=True):
-        self.child_nodes.append(newnode)
+    def append_child(self, new_node, prop_code=True):
+        """Append a node as a child.
+        prop_code -- propagate code to parent.
+        """
+        self.child_nodes.append(new_node)
         if prop_code:
 
             if not self.is_parsing:
-                self.append_code(newnode.code)
+                self.append_code(new_node.code)
 
-            if (newnode.code is not None and
-                    self.parent_node.node_name.endswith("-list") and
-                    self.parent_node.parent_node is not None and not self.parent_node.is_parsing and
+            if (new_node.code is not None and
+                    (self.parent_node.node_name.endswith("-list") or
+                     self.parent_node.node_name.endswith("-table")) and
+                    self.parent_node.parent_node is not None and
+                    not self.parent_node.is_parsing and
                     not self.parent_node.parent_node.is_parsing):
-                self.parent_node.parent_node.append_code(newnode.code)
+                self.parent_node.parent_node.append_code(new_node.code)
 
 
     def append_code(self, code):
+        """Extent the node's code and of its parents."""
         if self.code is None:
             if code is None:
                 self.code = code
@@ -168,6 +189,7 @@ class NodePartRST(Node):
 
 
     def prev_leaf(self):
+        """Return the previous leaf node."""
         node = self
         while not node.prev:
             if node.parent_node:
@@ -187,6 +209,7 @@ class NodePartRST(Node):
 
 
     def next_leaf(self):
+        """Return the next leaf node."""
         node = self
         while not node.next:
             if node.parent_node:
